# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,dcm)
S3method(as.data.frame,icm)
S3method(as.data.frame,netdx)
S3method(as.data.frame,netsim)
S3method(as.network,transmat)
S3method(as_cumulative_edgelist,networkDynamic)
S3method(comp_plot,dcm)
S3method(comp_plot,icm)
S3method(comp_plot,netsim)
S3method(get_network,netdx)
S3method(get_network,netsim)
S3method(get_network,netsim_dat)
S3method(merge,icm)
S3method(merge,netsim)
S3method(plot,dcm)
S3method(plot,epi.data.frame)
S3method(plot,icm)
S3method(plot,netdx)
S3method(plot,netsim)
S3method(plot,transmat)
S3method(print,control.dcm)
S3method(print,control.icm)
S3method(print,control.net)
S3method(print,dcm)
S3method(print,disscoef)
S3method(print,icm)
S3method(print,init.dcm)
S3method(print,init.icm)
S3method(print,init.net)
S3method(print,netdx)
S3method(print,netest)
S3method(print,netsim)
S3method(print,param.dcm)
S3method(print,param.icm)
S3method(print,param.net)
S3method(set_network,netsim_dat)
S3method(summary,dcm)
S3method(summary,icm)
S3method(summary,netest)
S3method(summary,netsim)
S3method(truncate_sim,dcm)
S3method(truncate_sim,icm)
S3method(truncate_sim,netsim)
export(add_attr)
export(add_control)
export(add_epi)
export(add_init)
export(add_param)
export(add_vertices)
export(append_attr)
export(append_core_attr)
export(apportion_lr)
export(arrivals.2g.net)
export(arrivals.icm)
export(arrivals.icm.bip)
export(arrivals.net)
export(arrive_nodes)
export(as.epi.data.frame)
export(as.phylo.transmat)
export(as_cumulative_edgelist)
export(as_tibble_edgelist)
export(auto_update_attr)
export(brewer_ramp)
export(check_degdist_bal)
export(color_tea)
export(comp_plot)
export(control.dcm)
export(control.icm)
export(control.net)
export(copy_datattr_to_nwattr)
export(copy_nwattr_to_datattr)
export(create_dat_object)
export(create_scenario_list)
export(crosscheck.dcm)
export(crosscheck.icm)
export(crosscheck.net)
export(dcm)
export(dedup_cumulative_edgelist)
export(delete_attr)
export(delete_edges)
export(delete_vertices)
export(depart_nodes)
export(departures.2g.net)
export(departures.icm)
export(departures.icm.bip)
export(departures.net)
export(discord_edgelist)
export(dissolution_coefs)
export(edgelist_censor)
export(edges_correct)
export(epiweb)
export(generate_random_params)
export(geom_bands)
export(get_args)
export(get_attr)
export(get_attr_history)
export(get_attr_list)
export(get_attr_prop)
export(get_backward_reachable)
export(get_control)
export(get_control_list)
export(get_core_attributes)
export(get_cumulative_degree)
export(get_cumulative_edgelist)
export(get_cumulative_edgelists_df)
export(get_current_timestep)
export(get_degree)
export(get_discordant_edgelist)
export(get_edgelist)
export(get_edgelists_df)
export(get_epi)
export(get_epi_list)
export(get_formula_term_attr)
export(get_forward_reachable)
export(get_init)
export(get_init_list)
export(get_network)
export(get_network_attributes)
export(get_network_control)
export(get_network_term_attr)
export(get_nwparam)
export(get_nwstats)
export(get_param)
export(get_param_list)
export(get_param_set)
export(get_partners)
export(get_posit_ids)
export(get_sims)
export(get_transmat)
export(get_unique_ids)
export(get_vertex_attribute)
export(icm)
export(idgroup)
export(increment_timestep)
export(infection.2g.net)
export(infection.icm)
export(infection.icm.bip)
export(infection.net)
export(init.dcm)
export(init.icm)
export(init.net)
export(init_nets)
export(init_status.icm)
export(init_status.net)
export(initialize.icm)
export(initialize.net)
export(is.transmat)
export(is_active_posit_ids)
export(is_active_unique_ids)
export(make_restart_point)
export(mcat)
export(mod_SIR_1g_cl)
export(mod_SIR_1g_op)
export(mod_SIR_2g_cl)
export(mod_SIR_2g_op)
export(mod_SIS_1g_cl)
export(mod_SIS_1g_op)
export(mod_SIS_2g_cl)
export(mod_SIS_2g_op)
export(mod_SI_1g_cl)
export(mod_SI_1g_op)
export(mod_SI_2g_cl)
export(mod_SI_2g_op)
export(multilayer)
export(mutate_epi)
export(netdx)
export(netest)
export(netsim)
export(network_initialize)
export(nwupdate.net)
export(overwrite_attrs)
export(padded_vector)
export(param.dcm)
export(param.icm)
export(param.net)
export(param.net_from_table)
export(param.net_to_table)
export(param_random)
export(prevalence.icm)
export(prevalence.icm.bip)
export(prevalence.net)
export(process_out.net)
export(record_attr_history)
export(record_raw_object)
export(recovery.2g.net)
export(recovery.icm)
export(recovery.icm.bip)
export(recovery.net)
export(remove_node_attr)
export(resim_nets)
export(saveout.dcm)
export(saveout.icm)
export(saveout.net)
export(set_attr)
export(set_control)
export(set_current_timestep)
export(set_epi)
export(set_init)
export(set_network)
export(set_param)
export(set_transmat)
export(set_vertex_attribute)
export(sim_nets_t1)
export(simulate_dat)
export(ssample)
export(summary_nets)
export(test_icm)
export(test_net)
export(trim_netest)
export(truncate_sim)
export(update_cumulative_edgelist)
export(update_dissolution)
export(update_params)
export(use_scenario)
export(verbose.dcm)
export(verbose.icm)
export(verbose.net)
import(ergm)
import(ggplot2)
import(network)
import(networkDynamic)
import(networkLite)
import(tergm)
importFrom(RColorBrewer,brewer.pal)
importFrom(RColorBrewer,brewer.pal.info)
importFrom(ape,as.phylo)
importFrom(ape,collapse.singles)
importFrom(coda,effectiveSize)
importFrom(deSolve,dede)
importFrom(deSolve,ode)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,rgb)
importFrom(graphics,abline)
importFrom(graphics,arrows)
importFrom(graphics,boxplot)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,polygon)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(lazyeval,lazy_dots)
importFrom(lazyeval,lazy_eval)
importFrom(methods,is)
importFrom(rlang,.data)
importFrom(statnet.common,.Deprecate_once)
importFrom(statnet.common,NVL)
importFrom(statnet.common,check.control.class)
importFrom(statnet.common,paste.and)
importFrom(statnet.common,set.control.class)
importFrom(statnet.common,trim_env)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,dgeom)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,rgeom)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,simulate)
importFrom(stats,supsmu)
importFrom(stats,terms.formula)
importFrom(stats,update)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(utils,head)
importFrom(utils,packageVersion)
importFrom(utils,tail)
useDynLib(EpiModel, .registration = TRUE)
