% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/net.mod.vital.R
\name{departures.2g.net}
\alias{departures.2g.net}
\title{Departures: netsim Module}
\usage{
departures.2g.net(dat, at)
}
\arguments{
\item{dat}{Main \code{netsim_dat} object containing a \code{networkDynamic}
object and other initialization information passed from
\code{\link[=netsim]{netsim()}}.}

\item{at}{Current time step.}
}
\value{
The updated \code{netsim_dat} main list object.
}
\description{
This function simulates departure for use in \link{netsim}
simulations.
}
\seealso{
\code{\link[=netsim]{netsim()}}
}
\keyword{internal}
\keyword{netMod}
